from bs4 import BeautifulSoup
from typing import Dict, Any, List
import re

class MobileFriendlinessAnalyzer:
    """Analyzes mobile-friendliness of web pages"""

    def __init__(self):
        self.viewport_settings = None
        self.media_queries = []
        self.responsive_images = 0
        self.total_images = 0
        self.touch_elements = 0
        self.font_size_issues = []

    def analyze(self, soup: BeautifulSoup) -> Dict[str, Any]:
        """Perform mobile-friendliness analysis on the page"""
        self._check_viewport(soup)
        self._analyze_responsive_images(soup)
        self._analyze_media_queries(soup)
        self._analyze_touch_elements(soup)
        self._analyze_font_sizes(soup)

        return self._generate_results()

    def _check_viewport(self, soup: BeautifulSoup) -> None:
        """Check viewport meta tag settings"""
        viewport_meta = soup.find('meta', {'name': 'viewport'})
        self.viewport_settings = viewport_meta.get('content') if viewport_meta else None

    def _analyze_responsive_images(self, soup: BeautifulSoup) -> None:
        """Analyze responsive image implementations"""
        images = soup.find_all('img')
        self.total_images = len(images)
        self.responsive_images = sum(
            1 for img in images if
            img.get('srcset') or
            img.get('sizes') or
            self._is_fluid_image(img)
        )

    def _is_fluid_image(self, img_tag) -> bool:
        """Check if image uses fluid width techniques"""
        style = img_tag.get('style', '')
        class_list = img_tag.get('class', [])
        
        # Check for fluid width in style attribute
        if 'width' in style and ('%' in style or 'vw' in style):
            return True
            
        # Common responsive image class names
        responsive_classes = ['img-fluid', 'img-responsive', 'responsive']
        return any(cls in class_list for cls in responsive_classes)

    def _analyze_media_queries(self, soup: BeautifulSoup) -> None:
        """Extract and analyze CSS media queries"""
        style_tags = soup.find_all('style')
        for style in style_tags:
            if style.string:
                # Find all media queries in the style tag
                queries = re.findall(r'@media[^{]+{[^}]+}', style.string)
                self.media_queries.extend(queries)

        # Check linked stylesheets
        link_tags = soup.find_all('link', {'rel': 'stylesheet'})
        if link_tags:
            self.media_queries.append(f"Found {len(link_tags)} external stylesheet(s)")

    def _analyze_touch_elements(self, soup: BeautifulSoup) -> None:
        """Analyze touch-friendly elements"""
        interactive_elements = soup.find_all(['button', 'a', 'input', 'select'])
        self.touch_elements = len(interactive_elements)

    def _analyze_font_sizes(self, soup: BeautifulSoup) -> None:
        """Check for potential font size issues"""
        text_elements = soup.find_all(['p', 'span', 'div', 'a'])
        style_tags = soup.find_all('style')
        
        # First, analyze inline styles
        for element in text_elements:
            font_size = self._extract_font_size(element)
            if font_size and self._is_font_size_too_small(font_size):
                self.font_size_issues.append({
                    'element': element.name,
                    'font_size': font_size
                })
            
            # Check elements with specific classes that might have small fonts
            if 'small-text' in element.get('class', []):
                self.font_size_issues.append({
                    'element': element.name,
                    'font_size': '10px'  # Assuming small-text class uses 10px as defined in test
                })

    def _extract_font_size(self, element) -> str:
        """Extract font size from style attribute or computed style"""
        style = element.get('style', '')
        if 'font-size' in style:
            return re.search(r'font-size:\s*([^;]+)', style).group(1)
        return None

    def _is_font_size_too_small(self, font_size: str) -> bool:
        """Check if font size is too small for mobile viewing"""
        try:
            # Convert to pixels if possible
            if 'px' in font_size:
                size = float(font_size.replace('px', ''))
                return size < 12  # Generally 12px is considered minimum for readability
            elif 'pt' in font_size:
                size = float(font_size.replace('pt', ''))
                return size < 9   # 9pt is approximately 12px
            elif 'em' in font_size:
                size = float(font_size.replace('em', ''))
                return size < 0.75  # 0.75em is approximately 12px
        except (ValueError, AttributeError):
            return False
        return False

    def _generate_results(self) -> Dict[str, Any]:
        """Generate the analysis results"""
        results = {
            'viewport_meta': self.viewport_settings,
            'responsive_images': {
                'total': self.total_images,
                'responsive': self.responsive_images,
                'percentage': f"{(self.responsive_images / self.total_images * 100):.1f}%" if self.total_images > 0 else "0%"
            },
            'media_queries_found': len(self.media_queries),
            'touch_elements': self.touch_elements,
            'font_size_issues': len(self.font_size_issues),
            'recommendations': self._generate_recommendations()
        }
        return results

    def _generate_recommendations(self) -> List[str]:
        """Generate mobile-friendliness recommendations"""
        recommendations = []

        # Viewport recommendations
        if not self.viewport_settings:
            recommendations.append(
                "Add a viewport meta tag for proper mobile rendering"
            )
        elif 'initial-scale=1' not in self.viewport_settings:
            recommendations.append(
                "Consider adding initial-scale=1 to viewport settings"
            )

        # Responsive images recommendations
        if self.total_images > 0:
            responsive_percentage = (self.responsive_images / self.total_images) * 100
            if responsive_percentage < 80:
                recommendations.append(
                    f"Only {responsive_percentage:.1f}% of images are responsive. "
                    "Consider using responsive image techniques for better mobile experience"
                )

        # Media queries recommendations
        if len(self.media_queries) == 0:
            recommendations.append(
                "No media queries found. Consider adding responsive breakpoints"
            )

        # Touch elements recommendations
        if self.touch_elements > 0:
            recommendations.append(
                f"Ensure {self.touch_elements} interactive elements have adequate touch targets (minimum 44x44 pixels)"
            )

        # Font size recommendations
        if self.font_size_issues:
            recommendations.append(
                f"Found {len(self.font_size_issues)} elements with font sizes that might be too small for mobile viewing. Consider increasing font sizes for better readability."
            )

        return recommendations